import { None, useEvent, World } from "@rbxts/matter"
import { PlayerCharacter } from "ReplicatedStorage/ecs/components"

/**
 * Equipped tool is replicated into the ECS.
 */
function playerCharacterToolIsTool(world: World): void {
	for (const [id, character] of world.query(PlayerCharacter)) {
        for (const [_, toolInstance] of useEvent(character.character, "ChildAdded")) {
            if (!classIs(toolInstance, "Tool")) continue

            world.insert(
                id,
                character.patch({
                    equippedTool: toolInstance
                })
            )
        }

        for (const [_, toolInstance] of useEvent(character.character, "ChildRemoved")) {
            // classIs more like classWas
            if (!classIs(toolInstance, "Tool")) continue

            // could also be swapping tools
            const swappedTool = character.character.FindFirstChildOfClass("Tool")

            world.insert(
                id,
                character.patch({
                    equippedTool: swappedTool ? swappedTool : None
                })
            )
        }
	}
}

export = playerCharacterToolIsTool